/******************************************************************************
**
**  Project Name:	DropShell
**     File Name:	DropShell.c
**
**   Description:	Main application code for the QuickShell
**
*******************************************************************************
**                       A U T H O R   I D E N T I T Y
*******************************************************************************
**
**	Initials	Name
**	--------	-----------------------------------------------
**	LDR			Leonard Rosenthol
**	MTC			Marshall Clow
**	SCS			Stephan Somogyi
**
*******************************************************************************
**                      R E V I S I O N   H I S T O R Y
*******************************************************************************
**
**	  Date		Author	Description
**	---------	------	---------------------------------------------
**	23 Jun	94	LDR		Implemented support for disk insertion events
**	02 Feb	94	LDR		Updated for Final SDK & CodeWarrior a2
**						Removed the ResumeProc as per new Apple recommendations
**	11 Dec 93	SCS		Universal Headers/UPPs (Phoenix 68k/PPC & PPCC)
**						Skipped System 6 compatible rev of DropShell source
**	09 Dec 91	LDR		Added support for new "Select File" menu item
**						Quit now sends AEVT to self to be politically correct
**						Added support for the new gSplashScreen
**	24 Nov 91	LDR		Added support for the Apple Menu (duh!)
**	29 Oct 91	SCS		Changes for THINK C 5
**	28 Oct 91	LDR		Officially renamed DropShell (from QuickShell)
**						Added a bunch of comments for clarification
**	06 Oct 91	MTC		Converted to MPW C
**	09 Apr 91	LDR		Added to Projector
**
******************************************************************************/
#include "GifScan.h"

#include "DSGlobals.h"
#include "DSUserProcs.h"
#include "DSAppleEvents.h"

#include "DropShell.h"

#include "Prefs.h"
#include "GSDrag.h"

Boolean		gDone, gOApped, gHasAppleEvents, gCmdHeldDown, gInBackground;
MenuHandle	gAppleMenu, gFileMenu, gEditMenu;
WindowPtr	gSplashScreen;

#ifdef MPW
extern void _DataInit();	
#endif

void InitToolbox (void) 
{

#ifdef MPW
	UnloadSeg ((Ptr) _DataInit );
#endif

	InitGraf ( &qd.thePort );
	InitFonts ();
	InitWindows ();
	InitMenus ();
	TEInit ();
	InitDialogs (NULL);		// use of ResumeProcs no longer approved by Apple
	InitCursor ();
	FlushEvents ( everyEvent, 0 );
	
	// how about some memory fun! Two should be enough!
	MoreMasters ();
	MoreMasters ();
	}

/*
	Let's setup those global variables that the DropShell uses.
	
	If you add any globals for your own use,
	init them in the InitUserGlobals routine in DSUserProcs.c
*/

Boolean InitGlobals(void) 
{
	long aLong;

	gDone			= false;
	gOApped			= false;	// probably not since users are supposed to DROP things!
	gHasAppleEvents	= Gestalt ( gestaltAppleEventsAttr, &aLong ) == noErr;
	gSplashScreen	= NULL;

	return(InitUserGlobals());	// call the user proc
}

/*
	Again, nothing fancy.  Just setting up the menus.
	
	If you add any menus to your DropBox - insert them here!
*/

void SetUpMenus (void) 
{
	gAppleMenu = GetMenu ( kAppleNum );
	AppendResMenu ( gAppleMenu, 'DRVR' );
	InsertMenu ( gAppleMenu, 0 );

	gFileMenu = GetMenu ( kFileNum );
	InsertMenu ( gFileMenu, 0 );
	
	gEditMenu = GetMenu ( kEditNum );
	InsertMenu(gEditMenu, 0);
	
	DrawMenuBar ();
}


/*	--------------- Standard Event Handling routines ---------------------- */

void ShowAbout ( void ) 
{
	(void) Alert ( 128, NULL );
	
}

void DoMenu ( long retVal ) 
{
	short			menuID, itemID;
	Str255			itemStr;
	OSErr			saveErr;
	TEHandle		te;
	WindowRef		window;
	Handle			aHandle;
	long			oldSize, newSize;
	long			total, contig;

	EventRecord		dummyEvent;

	menuID = HiWord ( retVal );
	itemID = LoWord ( retVal );
	
	switch ( menuID ) 
	{
		case kAppleNum:
			if ( itemID == 1 )
				ShowAbout ();	/*	Show the about box */
			else
			{
				GetMenuItemText(GetMenuHandle(kAppleNum), itemID, itemStr);
				OpenDeskAcc(itemStr);
			}
			break;
			
		case kFileNum:
			switch (itemID)
			{
				case 1:
					SelectFile();	// call file selection userProc
					break;
				
				case 2:
					if ( gOutWindow != nil )
					{
					 	 if ( FrontWindow() != nil ) 
					 	 	HideWindow(gOutWindow);
					 	 else 
					 	 	ShowWindow(gOutWindow);
					}
					break;
				
				case 3:	
					HandleTextFileSave();
					break;
					
				case 4:		/* Preferences */
					SetPreferences();
					break;
				
				default:
					SendQuitToSelf();	// send self a 'quit' event
					break;
			}
			break;
		
		case kEditNum:
			if ( !SystemEdit(menuID-1) ) 
			{
				te = ((DocumentPeek)FrontWindow())->docTE;
				switch ( itemID ) 
				{
					case 3:
						if ( ZeroScrap() == noErr ) 
						{
							PurgeSpace(&total, &contig);
							if ((*te)->selEnd - (*te)->selStart + kTESlop > contig)
								SysBeep(20);
							else 
							{
								TECut(te);
								if ( TEToScrap() != noErr ) 
								{
									SysBeep(20);
									ZeroScrap();
								}
							}
						}
						break;
					case 4:
						if ( ZeroScrap() == noErr ) 
						{
							TECopy(te);
							if ( TEToScrap() != noErr ) 
							{
								SysBeep(20);
								ZeroScrap();
							}
						}
						break;
						
					case 5:
						if ( TEFromScrap() == noErr ) 
						{
							if ( TEGetScrapLen() + ((*te)->teLength -
								((*te)->selEnd - (*te)->selStart)) > kMaxTELength )
								SysBeep(20);
							else 
							{
								aHandle = (Handle) TEGetText(te);
								oldSize = GetHandleSize(aHandle);
								newSize = oldSize + TEGetScrapLen() + kTESlop;
								SetHandleSize(aHandle, newSize);
								saveErr = MemError();
								SetHandleSize(aHandle, oldSize);
								if (saveErr != noErr)
									SysBeep(20);
								else
									TEPaste(te);
							}
						}
						else
							SysBeep(20);
						break;
						
					case 6:
						TEDelete(te);
						break;
					
					case 7:
						TESetSelect(0, kMaxTELength, te);
						break;
				}
			AdjustScrollbars(FrontWindow(), false);
			AdjustTE(FrontWindow());
			}
			break;
			
		default:
			break;
			
		}
	HiliteMenu(0);		// turn it off!
}

void DoMouseDown ( EventRecord *curEvent ) 
{	
	WindowRef		whichWindow;
	short			whichPart;
	RgnHandle		dragRgn;
	Point			mouse;
	DocumentPeek	doc;

	whichPart = FindWindow ( curEvent->where, &whichWindow );
	switch ( whichPart ) 
	{
		case inMenuBar:
			DoMenu( MenuSelect ( curEvent->where ));
			break;
		
		case inSysWindow:
			SystemClick( curEvent, whichWindow );
			break;
		
		case inContent:
			if ( whichWindow != FrontWindow() ) 
			{
				SelectWindow(whichWindow);
			}
			else
			{
				if ( whichWindow == gOutWindow )
				{
					doc = (DocumentPeek)whichWindow;
					TEGetHiliteRgn(doc->docHiliteRgn, doc->docTE);
					mouse = curEvent->where;
					GlobalToLocal(&mouse);
					if ( PtInRgn(mouse, doc->docHiliteRgn) )
					{
						if ( IsDragManagerInstalled() )
						{
							HandleDragSelection(curEvent, doc->docHiliteRgn);
						}
					}
					else
						DoContentClick(gOutWindow, curEvent);
				}
			}
			break;
				
		case inDrag:
			{
				Rect	boundsRect = (*GetGrayRgn())->rgnBBox;
				
				DragWindow ( whichWindow, curEvent->where, &boundsRect );
				AdjustDragRgn();
			}
			break;
		
		case inGoAway:
			if ( whichWindow == gOutWindow )
			{
				if ( TrackGoAway(gOutWindow, curEvent->where) )
				{
					HideWindow(gOutWindow);
					AdjustDragRgn();
				}
			}
			break;
		
		case inZoomIn:
		case inZoomOut:
			if ( whichWindow == gOutWindow )
			{
				if( TrackBox(gOutWindow, curEvent->where, whichPart) )
				{
					DoZoomWindow(gOutWindow, whichPart);
					AdjustDragRgn();
				}
			}
			break;
		
		case inGrow:
			if ( whichWindow == gOutWindow )
			{
				DoGrowWindow(gOutWindow, curEvent);
				AdjustDragRgn();
			}
			break;
			
		default:
			break;
		}
	}



void DoKeyDown(EventRecord *event)
{
	WindowPtr	window;
	char		key;
	TEHandle	te;

	window = FrontWindow();
	if ( IsAppWindow(window) ) 
	{
		te = ((DocumentPeek)window)->docTE;
		key = event->message & charCodeMask;

		if ( key == kDelChar || 
			(*te)->teLength - ((*te)->selEnd - (*te)->selStart) + 1 < kMaxTELength ) 
		{
			TEKey(key, te);
			AdjustScrollbars(window, false);
			AdjustTE(window);
		} 
		else
			SysBeep(20);  /* Exceeded TE 32k limit */
	}
}

void DoUpdate(WindowRef	window)
{
	if ( IsAppWindow(window) ) 
	{
		BeginUpdate(window);
		if ( ! EmptyRgn(window->visRgn) )
		{
			DrawWindow(window);
			AdjustDragRgn();
		}
		EndUpdate(window);
	}
}

void DoActivate(WindowRef window, Boolean becomingActive)
{
	RgnHandle	tempRgn, clipRgn;
	Rect		growRect;
	DocumentPeek doc;
	
	if ( IsAppWindow(window) ) 
	{
		doc = (DocumentPeek) window;
		if ( becomingActive ) 
		{
			tempRgn = NewRgn();
			clipRgn = NewRgn();
			GetLocalUpdateRgn(window, tempRgn);
			GetClip(clipRgn);
			DiffRgn(clipRgn, tempRgn, tempRgn);
			SetClip(tempRgn);
			TEActivate(doc->docTE);
			SetClip(clipRgn);
			DisposeRgn(tempRgn);
			DisposeRgn(clipRgn);
			
			(*doc->docVScroll)->contrlVis = kControlVisible;
			(*doc->docHScroll)->contrlVis = kControlVisible;
			InvalRect(&(*doc->docVScroll)->contrlRect);
			InvalRect(&(*doc->docHScroll)->contrlRect);

			growRect = window->portRect;

			growRect.top = growRect.bottom - kScrollbarAdjust;
			growRect.left = growRect.right - kScrollbarAdjust;
			InvalRect(&growRect);
			
			TEGetHiliteRgn(doc->docHiliteRgn, doc->docTE);
		}
		else
		{		
			TEDeactivate(doc->docTE);

			HideControl(doc->docVScroll);
			HideControl(doc->docHScroll);

			DrawGrowIcon(window);
		}
		AdjustDragRgn();
	}
}

static void AdjustCursor(void)
{
	WindowPtr	window;
	RgnHandle	iBeamRgn, dragArrowRgn;
	Rect		iBeamRect;
	Point		mouse;

	window = FrontWindow();
	if ( (! gInBackground) && (! IsDAWindow(window)) ) 
	{
		if ( IsAppWindow(window) ) 
		{
			SetPort(window);
			
			GetMouse(&mouse);

			iBeamRgn = NewRgn();
			dragArrowRgn = NewRgn();
			
			/* TEGetHiliteRgn get the region handle in LOCAL points */
			TEGetHiliteRgn(dragArrowRgn, ((DocumentPeek)window)->docTE);
			
			iBeamRect = (*((DocumentPeek)window)->docTE)->viewRect;
			RectRgn(iBeamRgn, &iBeamRect);
			
			if ( PtInRgn(mouse, dragArrowRgn) )
			{
				SetCursor(&qd.arrow);
			}
			else if ( PtInRgn(mouse, iBeamRgn) ) 
			{
				SetCursor(*GetCursor(iBeamCursor));
			}
			else
			{
				SetCursor(&qd.arrow);
			}
	
			DisposeRgn(dragArrowRgn);
			DisposeRgn(iBeamRgn);
		}
	}
}

static void DoIdle(void)
{
	WindowPtr	window;

	window = FrontWindow();
	if ( IsAppWindow(window) )
	{
		TEIdle(((DocumentPeek) window)->docTE);
		AdjustCursor();	// Always
	}
}

static void AdjustMenus(void)
{
	StringHandle		itemString;
	
	if ( FrontWindow() != nil )
	{
		itemString = GetString(130);
		SetMenuItemText(gFileMenu, 2, itemString[0]);
		SetItemCmd(gFileMenu, 2, 'W');
	}
	else
	{
		itemString = GetString(129);
		SetMenuItemText(gFileMenu, 2, itemString[0]);
		SetItemCmd(gFileMenu, 2, 'K');
	}
}

void AdjustDragRgn(void)
{
	DragInfoHandle	dragInfo;
	Rect			receiveRect;
	
	if ( IsDragManagerInstalled() )
	{
		dragInfo = (DragInfoHandle)GetWRefCon(gOutWindow);
		receiveRect = gOutWindow->portRect;
		receiveRect.bottom -= kScrollbarAdjust;
		receiveRect.right -= kScrollbarAdjust;
		RectRgn((*dragInfo)->receiveRgn, &receiveRect);
	}
}

void main ( void ) 
{
	char				key;
	EventRecord			event;
	
	InitToolbox ();
	
	gInBackground = false;
	
	if ( InitGlobals () ) 
	{	
		// if we succeeding in initting self
		if ( !gHasAppleEvents )
		{
			ErrorAlert ( kErrStringID, kCantRunErr, 0, true );
		}
		else
		{
			InitAEVTStuff ();
			SetUpMenus ();
			
			while ( !gDone ) 
			{
				if ( WaitNextEvent (everyEvent, &event, 0, nil) )
				{
					switch ( event.what ) 
					{
						case kHighLevelEvent:
							DoHighLevelEvent ( &event );
							break;
							
						case mouseDown:
							DoMouseDown ( &event );
							break;
							
						case keyDown:
						case autoKey:
							key = event.message & charCodeMask;
							if ( event.modifiers & cmdKey ) 
							{
								if ( event.what == keyDown ) 
								{
									DoMenu(MenuKey(key));
								}
							} 
							else
								DoKeyDown(&event);
							break;
						
						case kOSEvent:
							{
								switch ((event.message >> 24) & 0x0FF)
								{
									case kMouseMovedMessage:
										DoIdle();
										break;
										
									case kSuspendResumeMessage:	
										gInBackground = (event.message & kResumeMask) == 0;
										DoActivate(FrontWindow(), !gInBackground);
										break;
								}
							}
							break;
							
						case diskEvt:
							if (HiWord(event.message)) 
							{
								Point diskInitPt;
								
								diskInitPt.v = diskInitPt.h = 100;
								DILoad();
								DIBadMount(diskInitPt, event.message);
								DIUnload();
							}
							break;
						
						case updateEvt:
							DoUpdate((WindowRef)(event.message));
							break;
						
						case activateEvt:
							DoActivate((WindowRef)(event.message), (event.modifiers & activeFlag) != 0);
							break;
					}
					AdjustCursor();
					AdjustMenus();
				}
				else
				{
					DoIdle();
				}
			}
		}
	}
	DisposeUserGlobals();	// call the userproc to clean itself up
}